import { verticalTextAlignmentProperty } from '@nativescript-community/text';
import { themer } from '@nativescript-community/ui-material-core';
import { buttonColorProperty, counterMaxLengthProperty, digitsProperty, errorColorProperty, errorProperty, floatingColorProperty, floatingInactiveColorProperty, floatingProperty, helperColorProperty, helperProperty, strokeColorProperty, strokeDisabledColorProperty, strokeInactiveColorProperty } from '@nativescript-community/ui-material-core/textbase/cssproperties';
import { Background, Color, Font, Style, Utils, _updateCharactersInRangeReplacementString, backgroundInternalProperty, editableProperty, fontInternalProperty, paddingBottomProperty, paddingLeftProperty, paddingRightProperty, paddingTopProperty, placeholderColorProperty } from '@nativescript/core';
import { textProperty } from '@nativescript/core/ui/text-base';
import { TextFieldBase } from './textfield.common';
var TextInputControllerUnderlineImpl = /** @class */ (function (_super) {
    __extends(TextInputControllerUnderlineImpl, _super);
    function TextInputControllerUnderlineImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TextInputControllerUnderlineImpl.initWithOwner = function (owner) {
        var delegate = TextInputControllerUnderlineImpl.new();
        delegate._owner = new WeakRef(owner);
        return delegate;
    };
    TextInputControllerUnderlineImpl.prototype.textInsetsWithSizeThatFitsWidthHint = function (defaultValue, widthHint) {
        var result = _super.prototype.textInsetsWithSizeThatFitsWidthHint.call(this, defaultValue, widthHint);
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            result = owner._getTextInsetsForBounds(result);
        }
        return result;
    };
    return TextInputControllerUnderlineImpl;
}(MDCTextInputControllerUnderline));
var TextInputControllerImpl = /** @class */ (function (_super) {
    __extends(TextInputControllerImpl, _super);
    function TextInputControllerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TextInputControllerImpl.initWithOwner = function (owner) {
        var delegate = TextInputControllerImpl.new();
        delegate._owner = new WeakRef(owner);
        return delegate;
    };
    TextInputControllerImpl.prototype.textInsetsWithSizeThatFitsWidthHint = function (defaultValue, widthHint) {
        var result = _super.prototype.textInsetsWithSizeThatFitsWidthHint.call(this, defaultValue, widthHint);
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            result = owner._getTextInsetsForBounds(result);
        }
        return result;
    };
    return TextInputControllerImpl;
}(MDCTextInputControllerBase));
var TextInputControllerOutlinedImpl = /** @class */ (function (_super) {
    __extends(TextInputControllerOutlinedImpl, _super);
    function TextInputControllerOutlinedImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TextInputControllerOutlinedImpl.initWithOwner = function (owner) {
        var delegate = TextInputControllerOutlinedImpl.new();
        delegate._owner = new WeakRef(owner);
        return delegate;
    };
    TextInputControllerOutlinedImpl.prototype.textInsetsWithSizeThatFitsWidthHint = function (defaultValue, widthHint) {
        var result = _super.prototype.textInsetsWithSizeThatFitsWidthHint.call(this, defaultValue, widthHint);
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            result = owner._getTextInsetsForBounds(result);
        }
        return result;
    };
    return TextInputControllerOutlinedImpl;
}(MDCTextInputControllerOutlined));
var TextInputControllerFilledImpl = /** @class */ (function (_super) {
    __extends(TextInputControllerFilledImpl, _super);
    function TextInputControllerFilledImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TextInputControllerFilledImpl.initWithOwner = function (owner) {
        var delegate = TextInputControllerFilledImpl.new();
        delegate._owner = new WeakRef(owner);
        return delegate;
    };
    TextInputControllerFilledImpl.prototype.textInsetsWithSizeThatFitsWidthHint = function (defaultValue, widthHint) {
        var result = _super.prototype.textInsetsWithSizeThatFitsWidthHint.call(this, defaultValue, widthHint);
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            result = owner._getTextInsetsForBounds(result);
        }
        return result;
    };
    return TextInputControllerFilledImpl;
}(MDCTextInputControllerFilled));
export class TextField extends TextFieldBase {
    constructor() {
        super(...arguments);
        this.mCanAutoSize = false;
    }
    clearFocus() {
        this.dismissSoftInput();
    }
    requestFocus() {
        this.focus();
    }
    textFieldShouldChangeCharactersInRangeReplacementString(textField, range, replacementString) {
        // ignore if not in our alllowed digits
        if (this.nsdigits && replacementString.length > 0 && NSString.stringWithString(replacementString).rangeOfCharacterFromSet(this.nsdigits).location === NSNotFound) {
            return false;
        }
        if (this.secureWithoutAutofill && !textField.secureTextEntry) {
            /**
             * Helps avoid iOS 12+ autofill strong password suggestion prompt
             * Discussed in several circles but for example:
             * https://github.com/expo/expo/issues/2571#issuecomment-473347380
             */
            textField.secureTextEntry = true;
        }
        // we need to override this from N as in MDC case the range is 0
        // if (range.length > 0) {
        const delta = replacementString.length - range.length;
        if (delta > 0) {
            if (textField.text.length + delta > this.maxLength) {
                return false;
            }
        }
        // }
        if (this.updateTextTrigger === 'textChanged') {
            if (textField.secureTextEntry && this.firstEdit) {
                textProperty.nativeValueChange(this, replacementString);
            }
            else {
                if (range.location <= textField.text.length) {
                    const newText = NSString.stringWithString(textField.text).stringByReplacingCharactersInRangeWithString(range, replacementString);
                    textProperty.nativeValueChange(this, newText);
                }
            }
        }
        if (this.formattedText) {
            _updateCharactersInRangeReplacementString(this.formattedText, range.location, range.length, replacementString);
        }
        this.firstEdit = false;
        if (this.mCanAutoSize) {
            // if the textfield is in auto size we need to request a layout to take the new text width into account
            this.requestLayout();
        }
        return true;
        // return super.textFieldShouldChangeCharactersInRangeReplacementString(textField, range, replacementString);
    }
    onMeasure(widthMeasureSpec, heightMeasureSpec) {
        const widthMode = Utils.layout.getMeasureSpecMode(widthMeasureSpec);
        this.mCanAutoSize = widthMode !== Utils.layout.EXACTLY;
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
    }
    _getTextInsetsForBounds(insets) {
        const style = this.style;
        if (this.variant === 'underline' && this._controller.underlineHeightNormal === 0) {
            // if no underline/custom background, remove all insets like on android
            insets.top = 0;
            insets.bottom = 0;
        }
        if (paddingTopProperty.isSet(style)) {
            insets.top = Utils.layout.toDeviceIndependentPixels(this.effectivePaddingTop);
        }
        if (paddingRightProperty.isSet(style)) {
            insets.right = Utils.layout.toDeviceIndependentPixels(this.effectivePaddingRight);
        }
        if (paddingBottomProperty.isSet(style)) {
            insets.bottom = Utils.layout.toDeviceIndependentPixels(this.effectivePaddingBottom);
        }
        if (paddingLeftProperty.isSet(style)) {
            insets.left = Utils.layout.toDeviceIndependentPixels(this.effectivePaddingLeft);
        }
        return insets;
    }
    // variant = 'underline';
    createNativeView() {
        // const view = MDCTextFieldImpl.initWithOwner(new WeakRef(this));
        const view = MDCTextField.new();
        // disable it for now
        view.clearButtonMode = 0 /* UITextFieldViewMode.Never */;
        const scheme = MDCContainerScheme.new();
        scheme.colorScheme = themer.getAppColorScheme();
        if (this.style.variant === 'filled') {
            this._controller = TextInputControllerFilledImpl.initWithOwner(this);
        }
        else if (this.style.variant === 'outline') {
            this._controller = TextInputControllerOutlinedImpl.initWithOwner(this);
        }
        else if (this.style.variant === 'underline') {
            this._controller = TextInputControllerUnderlineImpl.initWithOwner(this);
        }
        else {
            this._controller = TextInputControllerImpl.initWithOwner(this);
            this._controller.floatingEnabled = false;
            // (this._controller as TextInputControllerImpl).applyThemeWithScheme(scheme);
        }
        this._controller.textInput = view;
        //theme needs to be applied after setting the textInput
        if (this.style.variant === 'filled') {
            this._controller.applyThemeWithScheme(scheme);
        }
        else if (this.style.variant === 'outline') {
            this._controller.applyThemeWithScheme(scheme);
        }
        else if (this.style.variant === 'underline') {
            this._controller.applyThemeWithScheme(scheme);
        }
        else {
            this._controller.underlineHeightActive = 0;
            this._controller.underlineHeightNormal = 0;
        }
        view.textInsetsMode = 1 /* MDCTextInputTextInsetsMode.IfContent */;
        // this._controller.
        // if (colorScheme) {
        //     MDCTextFieldColorThemer.applySemanticColorSchemeToTextInput(colorScheme, view);
        //     MDCTextFieldColorThemer.applySemanticColorSchemeToTextInputController(colorScheme, this._controller);
        // }
        return view;
    }
    // TODO: check why i was checking for isFirstResponder
    // with it the blur event is not fired anymore
    // public dismissSoftInput() {
    //     if (this.nativeViewProtected.isFirstResponder) {
    //         super.dismissSoftInput();
    //     }
    // }
    blur() {
        this.dismissSoftInput();
    }
    setSelection(start, stop) {
        const view = this.nativeTextViewProtected;
        if (stop !== undefined) {
            const begin = view.beginningOfDocument;
            view.selectedTextRange = view.textRangeFromPositionToPosition(view.positionFromPositionOffset(begin, start), view.positionFromPositionOffset(begin, stop));
        }
        else {
            const begin = view.beginningOfDocument;
            const pos = view.positionFromPositionOffset(begin, start);
            view.selectedTextRange = view.textRangeFromPositionToPosition(pos, pos);
        }
    }
    [editableProperty.setNative](value) {
        this.clearFocus();
        // this.nativeTextViewProtected.enabled = value;
    }
    [floatingColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.floatingPlaceholderActiveColor = color;
        this._updateAttributedPlaceholder();
    }
    [floatingInactiveColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        // this._controller.inlinePlaceholderColor = color;
        this._controller.floatingPlaceholderNormalColor = color;
        this._updateAttributedPlaceholder();
    }
    [placeholderColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.inlinePlaceholderColor = color;
        if (!this.floatingColor) {
            this._controller.floatingPlaceholderActiveColor = color;
        }
        this._updateAttributedPlaceholder();
    }
    [errorColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.errorColor = color;
    }
    [strokeColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.activeColor = color;
    }
    [strokeInactiveColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.normalColor = color;
    }
    [strokeDisabledColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.disabledColor = color;
    }
    [buttonColorProperty.setNative](value) {
        const color = value instanceof Color ? value.ios : value;
        this._controller.textInputClearButtonTintColor = color;
    }
    [helperProperty.setNative](value) {
        this._controller.helperText = value;
    }
    [helperColorProperty.setNative](value) {
        const temp = typeof value === 'string' ? new Color(value) : value;
        const color = temp.ios;
        this._controller.leadingUnderlineLabelTextColor = color;
    }
    [counterMaxLengthProperty.setNative](value) {
        this._controller.characterCountMax = value;
    }
    [floatingProperty.setNative](value) {
        this._controller.floatingEnabled = value;
    }
    [errorProperty.setNative](value) {
        this._controller.setErrorTextErrorAccessibilityValue(value, value);
    }
    [digitsProperty.setNative](value) {
        if (value && value.length > 0) {
            this.nsdigits = NSCharacterSet.characterSetWithCharactersInString(value);
        }
        else {
            this.nsdigits = null;
        }
    }
    [backgroundInternalProperty.setNative](value) {
        switch (this.variant) {
            case 'none':
                super[backgroundInternalProperty.setNative](value);
                break;
            case 'outline':
            case 'filled':
            case 'underline': {
                if (value.color) {
                    this._controller.borderFillColor = value.color.ios;
                }
                if (value.borderTopColor) {
                    this._controller.activeColor = value.borderTopColor.ios;
                }
                break;
            }
        }
    }
    [fontInternalProperty.setNative](value) {
        super[fontInternalProperty.setNative](value);
        const font = value instanceof Font ? value.getUIFont(this._controller.inlinePlaceholderFont) : value;
        this._controller.inlinePlaceholderFont = font;
    }
    [verticalTextAlignmentProperty.setNative](value) {
        // TODO: not working for now
        const view = this.nativeTextViewProtected;
        view.backgroundColor = UIColor.redColor;
        switch (value) {
            case 'initial':
            case 'top':
                view.contentVerticalAlignment = 1 /* UIControlContentVerticalAlignment.Top */;
                break;
            case 'middle':
                view.contentVerticalAlignment = 0 /* UIControlContentVerticalAlignment.Center */;
                break;
            case 'bottom':
                view.contentVerticalAlignment = 2 /* UIControlContentVerticalAlignment.Bottom */;
                break;
        }
    }
}
//# sourceMappingURL=textfield.ios.old.js.map